import logging
import os

import aftership

logger = logging.getLogger("aftership")


def _build_tracking_url(tracking_id, slug, tracking_number):
    if tracking_id and (slug or tracking_number):
        raise ValueError('Cannot specify tracking number and tracking id at the same time')
    elif tracking_id:
        url = '{}'.format(tracking_id)
    elif slug and tracking_number:
        url = '{}/{}'.format(slug, tracking_number)
    else:
        raise ValueError('You must specify the tracking number of tracking id')
    return url


def get_api_key():
    """Get AfterShip API key"""
    if aftership.api_key is not None:
        return aftership.api_key
    return os.getenv('AFTERSHIP_API_KEY')



class AfterShipError(Exception):
    def __init__(self, message=None, code=None,
                 http_status=None, response=None,
                 ):
        super(AfterShipError, self).__init__()
        self.message = message
        self.code = code
        self.http_status = http_status
        self.response = response

    def __str__(self):
        return '{}: {}'.format(self.__class__.__name__, self.message)


class BadRequest(AfterShipError):
    pass


class Unauthorized(AfterShipError):
    pass


class Forbidden(AfterShipError):
    pass


class NotFound(AfterShipError):
    pass


class TooManyRequests(AfterShipError):
    pass


class InternalError(AfterShipError):
    pass


class UnexpectedError(AfterShipError):
    pass



from .exception import BadRequest, Forbidden, NotFound, Unauthorized, TooManyRequests, InternalError, UnexpectedError

error_mapping = {
    'BadRequest': BadRequest,
    'Unauthorized': Unauthorized,
    'Forbidden': Forbidden,
    'NotFound': NotFound,
    'TooManyRequests': TooManyRequests,
    'InternalError': InternalError,
}


def process_response(response):
    json_content = response.json()

    if response.status_code // 100 == 2:
        return json_content['data']

    error_type = json_content['meta']['type']

    if error_type in error_mapping:
        error_cls = error_mapping[error_type]
    else:
        error_cls = UnexpectedError

    raise error_cls(message=json_content['meta']['message'],
                    code=json_content['meta']['code'],
                    http_status=response.status_code,
                    response=response,
                    )



from urllib.parse import urljoin

import requests

from .const import API_KEY_FILED_NAME, API_ENDPOINT
from .util import get_api_key


def build_request_url(path):
    return urljoin(API_ENDPOINT, path)


def make_request(method, path, **kwargs):
    url = build_request_url(path)

    headers = kwargs.pop('headers', dict())
    if headers.get(API_KEY_FILED_NAME) is None:
        headers[API_KEY_FILED_NAME] = get_api_key()
    kwargs['headers'] = headers
    return requests.request(method, url, **kwargs)



from .request import make_request
from .response import process_response
from .util import _build_tracking_url


def create_tracking(tracking, **kwargs):
    """Create a tracking.
    """
    response = make_request('POST', 'trackings', json=dict(tracking=tracking), **kwargs)
    return process_response(response)


def get_tracking(*, tracking_id=None, slug=None, tracking_number=None, **kwargs):
    """Create a tracking.
    """
    optional_keys = ('fields', 'lang')
    params = {key: kwargs.pop(key) for key in optional_keys if key in kwargs}
    url = 'trackings/{}'.format(_build_tracking_url(tracking_id, slug, tracking_number))
    response = make_request('GET', url, params=params, **kwargs)
    return process_response(response)


def update_tracking(*, tracking, tracking_id=None, slug=None, tracking_number=None, **kwargs):
    """Update a tracking.
    """
    url = 'trackings/{}'.format(_build_tracking_url(tracking_id, slug, tracking_number))
    response = make_request('PUT', url, json=dict(tracking=tracking), **kwargs)
    return process_response(response)


def retrack(*, tracking_id=None, slug=None, tracking_number=None, **kwargs):
    """Retrack an expired tracking. Max 3 times per tracking.
    """
    url = 'trackings/{}/retrack'.format(_build_tracking_url(tracking_id, slug, tracking_number))
    response = make_request('POST', url, **kwargs)
    return process_response(response)


def get_last_checkpoint(*, tracking_id=None, slug=None, tracking_number=None, **kwargs):
    """Return the tracking information of the last checkpoint of a single tracking.
    """
    url = 'last_checkpoint/{}'.format(_build_tracking_url(tracking_id, slug, tracking_number))
    response = make_request('GET', url, **kwargs)
    return process_response(response)


def delete_tracking(*, tracking_id=None, slug=None, tracking_number=None, **kwargs):
    """Delete a tracking.
    """
    url = 'trackings/{}'.format(_build_tracking_url(tracking_id, slug, tracking_number))
    response = make_request('DELETE', url, **kwargs)
    return process_response(response)


def list_trackings(**kwargs):
    """Get tracking results of multiple trackings.
    """
    optional_keys = ('page', 'limit', 'keyword', 'slug', 'delivery_time', 'origin',
                     'destination', 'tag', 'created_at_min', 'created_at_max',
                     'fields', 'lang')

    params = {key: kwargs.pop(key) for key in optional_keys if key in kwargs}

    response = make_request('GET', 'trackings', params=params, **kwargs)
    return process_response(response)



from .request import make_request
from .response import process_response

__all__ = ['list_couriers', 'list_all_couriers', 'detect_courier']


def list_couriers(**kwargs):
    """Return a list of couriers activated at your AfterShip account.
    """
    response = make_request('GET', 'couriers', **kwargs)
    return process_response(response)


def list_all_couriers(**kwargs):
    """Return a list of all couriers.
    """
    response = make_request('GET', 'couriers/all', **kwargs)
    return process_response(response)


def detect_courier(tracking, **kwargs):
    """Return a list of matched couriers based on tracking number format and
    selected couriers or a list of couriers.
    """
    response = make_request('POST', 'couriers/detect', json=dict(tracking=tracking), **kwargs)
    return process_response(response)



import codecs
import os
import re
from codecs import open  # To use a consistent encoding

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    print(version_file)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


# Get the long description from the relevant file
with open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

with open(os.path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    install_requires = f.read().split('\n')

__version__ = find_version("aftership", "__init__.py")

setup(
    name='aftership',

    python_requires='>=3.5.0',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # http://packaging.python.org/en/latest/tutorial.html#version
    version=__version__,

    description='Python SDK of AfterShip API',
    long_description=long_description,

    # The project's main homepage.
    url='https://github.com/AfterShip/aftership-python',

    # Download path
    download_url='https://github.com/AfterShip/aftership-python/tarball/{}'.format(__version__),

    # Author details
    author='AfterShip',
    author_email='support@aftership.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 5 - Production/Stable',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],

    # What does your project relate to?
    keywords='aftership api binding tracking track post mail airmail logistics shipping',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=['aftership'],
    install_requires=install_requires,
)



from .request import make_request
from .response import process_response
from .util import _build_tracking_url


def list_notifications(*, tracking_id=None, slug=None, tracking_number=None, **kwargs):
    """ Get contact information for the users to notify when the tracking changes.
    Please note that only customer receivers will be returned. Any `email`, `sms`
    or `webhook` that belongs to the Store will not be returned.
    """
    url = 'notifications/{}'.format(_build_tracking_url(tracking_id, slug, tracking_number))
    response = make_request('GET', url, **kwargs)
    return process_response(response)


def add_notification(*, notification, tracking_id=None, slug=None, tracking_number=None, **kwargs):
    """Add notification receivers to a tracking number.
    """
    url = 'notifications/{}/add'.format(_build_tracking_url(tracking_id, slug, tracking_number))
    response = make_request('POST', url, json=dict(notification=notification), **kwargs)
    return process_response(response)


def remove_notification(*, notification, tracking_id=None, slug=None, tracking_number=None, **kwargs):
    """Remove notification receivers from a tracking number.
    """
    url = 'notifications/{}/remove'.format(_build_tracking_url(tracking_id, slug, tracking_number))
    response = make_request('POST', url, json=dict(notification=notification), **kwargs)
    return process_response(response)
